# Copyright (c) 2002-2004 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""A generic function to get a database connection.
"""

__revision__ = "$Id: db.py,v 1.18 2004/04/22 08:48:06 syt Exp $"

import sys

class UnknownDriver(Exception):
    """raised when a unknown driver is given to get connexion"""

def _psyco_parms(host, database, user, password):
    if host:
        dburi = 'host=%s  dbname=%s  user=%s' % (host, database, user)
    else:
        dburi = 'dbname=%s  user=%s' % (database, user)
    if password:
        dburi = '%s password=%s' % (dburi, password)
    return (dburi,), {}

def _standard_parms(host, database, user, password):
    return (), {'host':host, 'database':database,
                'user':user, 'password':password}

def _pypgsql_parms(host, database, user, password):
    return (), {'host':host, 'database':database,
                'user':user, 'password':password or None}

def _psyco_post(connection):
    connection.set_isolation_level(1)

DRIVERS = {"postgres":[('psycopg', (_psyco_parms, _psyco_post)),
                       ('pgdb', (_standard_parms, None)),
                       ('pyPgSQL.PgSQL', (_pypgsql_parms, None))],
           "mysql": [("pyMySQL.MySQL", (_standard_parms, None))],
           }


def set_prefered_driver(database, module, syntax = None,
                        _drivers=DRIVERS):
    """sets the prefered driver module for database
    database is the name of the db engine (postgresql, mysql...)
    module is the name of the module providing the connect function
    syntax is (params_func, post_process_func_or_None)
    _drivers is a optionnal dictionnary of drivers
    """
    try:
        modules = _drivers[database]
    except KeyError:
        raise UnknownDriver('Unknown database %s' % database)
    for mod, old_syntax in modules:
        if module == mod:
            modules.remove((mod, old_syntax))
            if syntax is None:
                syntax = old_syntax
            modules.insert(0, (mod, syntax))
            break
    else:
        raise UnknownDriver('Unknown module %s for %s' % (module, database))

def get_connexion(driver='postgres', host='', database='',
                  user='', password='', quiet=0, drivers = DRIVERS):
    """return a db connexion according to given arguments"""
    for mod, (syntax, postprocess) in drivers[driver]:
        try:
            if not quiet:
                print >> sys.stderr, 'Trying %s'%mod
            connect = __import__(mod, globals(), locals(), ['connect']).connect
            break
        except ImportError:
            if not quiet:
                print >> sys.stderr, '%s is not available' % mod
    else:
        raise ImportError('Unable to import a %s module' % driver)
    
    args, kwargs = syntax(host, database, user, password)
    cnx = connect(*args, **kwargs)
    if postprocess is not None:
        postprocess(cnx)
    return cnx

def binary(driver='postgres', drivers=DRIVERS):
    """return the binary wrapper for the given driver"""
    for mod, (syntax, postprocess) in drivers[driver]:
        try:
            module = __import__(mod, globals(), locals(),['connect'])
            break
        except ImportError:
            continue
    else:
        raise ImportError('Unable to import a %s module' % driver)
    return module.Binary
    
def system_database(driver='postgres'):
    """return the system database for the given driver"""
    if driver == 'postgres':
        return 'template1'
    raise UnknownDriver('Unknown driver %s' % driver)
