"""
unit tests for module logilab.common.db

"""
__revision__ = "$Id: unittest_db.py,v 1.2 2004/02/20 12:58:29 syt Exp $"

import unittest
from logilab.common.db import *

class PreferedDriverTC(unittest.TestCase):
    def setUp(self):
        self.drivers = {"pg":[('foo', None), ('bar', None)]}
        
    def testNormal(self):
        set_prefered_driver('pg','bar', None,  self.drivers)
        self.assertEquals('bar', self.drivers['pg'][0][0])
        set_prefered_driver('pg','bar', 'Fnurb',  self.drivers)
        self.assertEquals(('bar', 'Fnurb'), self.drivers['pg'][0])
    
    def testFailuresDb(self):
        try:
            set_prefered_driver('oracle','bar', None, self.drivers)
            self.fail()
        except UnknownDriver, exc:
            self.assertEquals(exc.args[0], 'Unknown database oracle')

    def testFailuresDriver(self):
        try:
            set_prefered_driver('pg','baz', None, self.drivers)
            self.fail()
        except UnknownDriver, exc:
            self.assertEquals(exc.args[0], 'Unknown module baz for pg')

    def testGlobalVar(self):
        old_drivers = DRIVERS['postgres'][:]
        expected = old_drivers[:]
        expected.insert(0, expected.pop(1))
        set_prefered_driver('postgres','pgdb')
        self.assertEquals(DRIVERS['postgres'], expected)
        set_prefered_driver('postgres','psycopg')
        self.assertEquals(DRIVERS['postgres'], old_drivers)

class getCnxTC(unittest.TestCase):
    def setUp(self):
        self.host = 'crater.logilab.fr'
        self.db = 'bugtracker'
        self.user = 'adim'
        self.passwd = 'adim'
        
    def testPsyco(self):
        set_prefered_driver('postgres', 'psycopg')
        try:
            cnx = get_connexion('postgres',
                                self.host, self.db, self.user, self.passwd,
                                quiet=1)
        except ImportError:
            self.fail('python-psycopg is not installed')

    def testPgdb(self):
        set_prefered_driver('postgres', 'pgdb')
        try:
            cnx = get_connexion('postgres',
                                self.host, self.db, self.user, self.passwd,
                                quiet=1)
        except ImportError:
            self.fail('python-pgsql is not installed')

    def testPgsql(self):
        set_prefered_driver('postgres', 'pyPgSQL.PgSQL')
        try:
            cnx = get_connexion('postgres',
                                self.host, self.db, self.user, self.passwd,
                                quiet=1)
        except ImportError:
            self.fail('python-pygresql is not installed')

    def testMysql(self):
        set_prefered_driver('mysql', 'pyMySQL.MySQL')
        try:
            cnx = get_connexion('mysql',
                                self.host, self.db, self.user, self.passwd,
                                quiet=1)
        except ImportError:
            self.fail('python-mysql is not installed')

        
if __name__ == '__main__':
    unittest.main()
